"""
    VMC vCenter API Request Module
"""
import logging

import requests
from requests.auth import HTTPBasicAuth
from requests.exceptions import HTTPError
from requests.exceptions import RequestException
from requests.exceptions import SSLError
from saltext.vmware.utils import vmc_constants
from saltext.vmware.utils import vmc_request

log = logging.getLogger(__name__)


def get_api_session_id(hostname, username, password):
    """
    This function returns api_session_id required to perform operations for VMC vCenter.

    hostname
        Hostname of the vCenter console

    username
        username required to login to vCenter console

    password
        password required to login to vCenter console

    """
    url = vmc_request.set_base_url(hostname) + vmc_constants.VCENTER_API_SESSION_URL
    headers = {vmc_constants.CONTENT_TYPE: vmc_constants.APPLICATION_JSON}
    response = requests.post(url, headers=headers, auth=HTTPBasicAuth(username, password))
    return response.json()


def get_headers(hostname, username, password):
    """
    This function returns HTTP headers required to perform operations for VMC vCenter.

    hostname
        Hostname of the vCenter console

    username
        username required to login to vCenter console

    password
        password required to login to vCenter console

    """
    api_session_id = get_api_session_id(hostname, username, password)
    return {
        vmc_constants.CONTENT_TYPE: vmc_constants.APPLICATION_JSON,
        vmc_constants.VMWARE_API_SESSION_ID: api_session_id,
    }


def call_api(
    method,
    url,
    headers,
    description,
    responsebody_applicable=True,
    verify_ssl=True,
    cert=None,
    data=None,
    params=None,
):
    """
    This function is used to make the http requests for the given operation on vCenter and return its response

    method
        http request method : post, get, patch, put and delete

    url
        url to perform the operation

    headers
        headers required to perform the given operation

    description
        indicates the operation for which this function gets called. <module>.<function_name>

    responsebody_applicable
        boolean value which indicates if the requested api returns response body or not. Enabled by Default.

    verify_ssl
        Option to enable/disable SSL verification. Enabled by default.
        If set to False, the certificate validation is skipped.

    cert
        Path to the SSL certificate file to connect to VMC Cloud Console.
        The certificate can be retrieved from browser.

    data
        payload required for post and patch operations.

    params
        query params required to perform the given operation.

    """

    session = requests.Session()
    verify = verify_ssl
    if verify_ssl:
        if cert:
            verify = cert
        else:
            return {vmc_constants.ERROR: vmc_constants.NO_CERTIFICATE_ERROR_MSG}

    try:
        response = session.request(
            method=method, url=url, headers=headers, verify=verify, json=data, params=params
        )

        log.info("Response status code: %s for: %s", response.status_code, description)
        # raise error for any client/server HTTP Error codes
        response.raise_for_status()

        if not responsebody_applicable:
            return {"description": description, "result": "success"}
        return response.json()

    except HTTPError as e:
        log.error(e)
        result = {vmc_constants.ERROR: vmc_constants.HTTP_ERROR_MSG.format(url, description)}
        # if response contains json, extract error message from it
        if e.response.text:
            log.error("Response from VMC vCenter %s for %s", e.response.text, description)
            try:
                error_json = e.response.json()
                result[vmc_constants.ERROR] = error_json
            except ValueError:
                log.error(vmc_constants.PARSE_ERROR_MSG)
                result[vmc_constants.ERROR] = e.response.text
        return result
    except SSLError as se:
        log.error(se)
        result = {vmc_constants.ERROR: vmc_constants.SSL_ERROR_MSG.format(url, description)}
        return result
    except RequestException as re:
        log.error(re)
        result = {vmc_constants.ERROR: vmc_constants.REQUEST_EXCEPTION_MSG.format(url, description)}
        return result



# Copyright 2021 VMware, Inc.
# SPDX-License: Apache-2.0
import logging

import salt.exceptions
import saltext.vmware.utils.common as utils_common
import saltext.vmware.utils.connect as connect

log = logging.getLogger(__name__)

try:
    from pyVmomi import vim

    HAS_PYVMOMI = True
except ImportError:
    HAS_PYVMOMI = False


__virtualname__ = "vmware_tag"
__func_alias__ = {"list_": "list"}


def __virtual__():
    if not HAS_PYVMOMI:
        return False, "Unable to import pyVmomi module."
    return __virtualname__


def create(tag_name, category_id, description=""):
    """
    Create a new tag.

    tag_name
        Name of tag.

    category_id
        (string) Category ID of type: com.vmware.cis.tagging.Tag.

    description
        (optional) Description for the tag being created.
    """
    data = {
        "create_spec": {"category_id": category_id, "description": description, "name": tag_name}
    }
    response = connect.request(
        "/rest/com/vmware/cis/tagging/tag", "POST", body=data, opts=__opts__, pillar=__pillar__
    )
    response = response["response"].json()
    return response["value"]


def get(tag_id):
    """
    Returns info on given tag.

    tag_id
        (string) Tag ID of type: com.vmware.cis.tagging.Tag.
    """
    url = f"/rest/com/vmware/cis/tagging/tag/id:{tag_id}"
    response = connect.request(url, "GET", opts=__opts__, pillar=__pillar__)
    response = response["response"].json()
    return response["value"]


def update(tag_id, tag_name=None, description=None):
    """
    Updates give tag.

    tag_id
        (string) Tag ID of type: com.vmware.cis.tagging.Tag.

    tag_name
        Name of tag.

    description
        (optional) Description for the tag being created.
    """
    spec = {"update_spec": {}}
    if tag_name:
        spec["update_spec"]["name"] = tag_name
    if description:
        spec["update_spec"]["description"] = description
    url = f"/rest/com/vmware/cis/tagging/tag/id:{tag_id}"
    response = connect.request(url, "PATCH", body=spec, opts=__opts__, pillar=__pillar__)
    if response["response"].status_code == 200:
        return "updated"
    return {
        "tag": "failed to update",
        "status_code": response["response"].status_code,
        "reason": response["response"].reason,
    }


def delete(tag_id):
    """
    Delete given tag.

    tag_id
        (string) Tag ID of type: com.vmware.cis.tagging.Tag.
    """
    url = f"/rest/com/vmware/cis/tagging/tag/id:{tag_id}"
    response = connect.request(url, "DELETE", opts=__opts__, pillar=__pillar__)
    if response["response"].status_code == 200:
        return "deleted"
    return {
        "tag": "failed to update",
        "status_code": response.status_code,
        "reason": response.reason,
    }


def list_():
    """
    Lists IDs for all the tags on a given vCenter.
    """
    response = connect.request(
        "/rest/com/vmware/cis/tagging/tag", "GET", opts=__opts__, pillar=__pillar__
    )
    response = response["response"].json()
    return response["value"]


def list_category():
    """
    Lists IDs for all the categories on a given vCenter.
    """
    response = connect.request(
        "/rest/com/vmware/cis/tagging/category", "GET", opts=__opts__, pillar=__pillar__
    )
    response = response["response"].json()
    return response["value"]


def get_category(category_id):
    """
    Returns info on given category.

    category_id
        (string) Category ID of type: com.vmware.cis.tagging.Category.
    """
    url = f"/rest/com/vmware/cis/tagging/category/id:{category_id}"
    response = connect.request(url, "GET", opts=__opts__, pillar=__pillar__)
    response = response["response"].json()
    return response["value"]


def create_category(category_name, associable_types, cardinality, description=""):
    """
    Create a new category.

    category_name
        The display name of the category.

    associable_types
        (list) Object types to which this category’s tags can be attached.

    cardinality
        The CategoryModel.Cardinality enumerated type defines the number of tags in a category that can be assigned to an object. SINGLE, MULTIPLE

    description
        (optional) The description of the category.
    """
    data = {
        "create_spec": {
            "associable_types": associable_types,
            "cardinality": cardinality,
            "description": description,
            "name": category_name,
        }
    }
    response = connect.request(
        "/rest/com/vmware/cis/tagging/category", "POST", body=data, opts=__opts__, pillar=__pillar__
    )
    response = response["response"].json()
    return response["value"]


def update_category(
    category_id, category_name=None, associable_types=None, cardinality=None, description=""
):
    """
    Update a new category.

    category_id
        The identifier of the category to be updated. The parameter must be an identifier for the resource type: com.vmware.cis.tagging.Category.

    category_name
        The display name of the category.

    associable_types
        (list) Object types to which this categorys tags can be attached.

    cardinality
        The CategoryModel.Cardinality enumerated type defines the number of tags in a category that can be assigned to an object. SINGLE, MULTIPLE

    description
        (optional) The description of the category.
    """
    spec = {"update_spec": {}}
    if category_name:
        spec["update_spec"]["name"] = category_name
    if associable_types:
        spec["update_spec"]["associable_types"] = associable_types
    if cardinality:
        spec["update_spec"]["cardinality"] = cardinality
    if description:
        spec["update_spec"]["description"] = description
    url = f"/rest/com/vmware/cis/tagging/category/id:{category_id}"
    response = connect.request(url, "PATCH", body=spec, opts=__opts__, pillar=__pillar__)
    if response["response"].status_code == 200:
        return "updated"
    return {
        "category": "failed to update",
        "status_code": response["response"].status_code,
        "reason": response["response"].reason,
    }


def delete_category(category_id):
    """
    Delete given category.

    category_id
        The identifier of category to be deleted. The parameter must be an identifier for the resource type: com.vmware.cis.tagging.Category.
    """
    url = f"/rest/com/vmware/cis/tagging/category/id:{category_id}"
    response = connect.request(url, "DELETE", opts=__opts__, pillar=__pillar__)
    if response["response"].status_code == 200:
        return "deleted"
    return {
        "category": "failed to update",
        "status_code": response.status_code,
        "reason": response.reason,
    }



"""
Salt Module to perform CRUD operations for NSX-T's IP Address Blocks
"""
import logging

from saltext.vmware.utils import common
from saltext.vmware.utils import nsxt_request

log = logging.getLogger(__name__)

__virtual_name__ = "nsxt_ip_blocks"

IP_BLOCKS_BASE_URL = "https://{}/api/v1/pools/ip-blocks"


def __virtual__():
    return __virtual_name__


def get(
    hostname,
    username,
    password,
    verify_ssl=True,
    cert=None,
    cert_common_name=None,
    cursor=None,
    included_fields=None,
    page_size=None,
    sort_by=None,
    sort_ascending=None,
):
    """
    Lists IP Address blocks present in the NSX-T Manager for given query params

    CLI Example:

    .. code-block:: bash

        salt vm_minion nsxt_ip_blocks.get hostname=nsxt-manager.local username=admin ...

    hostname
        The host name of NSX-T manager

    username
        Username to connect to NSX-T manager

    password
        Password to connect to NSX-T manager

    verify_ssl
        (Optional) Option to enable/disable SSL verification. Enabled by default.
        If set to False, the certificate validation is skipped.

    cert
        (Optional) Path to the SSL client certificate file to connect to NSX-T manager.
        The certificate can be retrieved from browser.

    cert_common_name
        (Optional) By default, the hostname parameter and the common name in certificate is compared for host name
        verification. If the client certificate common name and hostname do not match (in case of self-signed
        certificates), specify the certificate common name as part of this parameter. This value is then used to
        compare against

    cursor
        (Optional) Opaque cursor to be used for getting next page of records (supplied by current result page)

    included_fields
        (Optional) Comma separated list of fields that should be included in query result

    page_size
        (Optional) Maximum number of results to return in this page

    sort_by
        (Optional) Field by which records are sorted

    sort_ascending
        (Optional) Boolean value to sort result in ascending order

    """

    log.info("Fetching IP Address Blocks")
    url = IP_BLOCKS_BASE_URL.format(hostname)

    params = common._filter_kwargs(
        allowed_kwargs=["cursor", "included_fields", "page_size", "sort_ascending", "sort_by"],
        default_dict=None,
        cursor=cursor,
        included_fields=included_fields,
        page_size=page_size,
        sort_by=sort_by,
        sort_ascending=sort_ascending,
    )

    return nsxt_request.call_api(
        method="get",
        url=url,
        username=username,
        password=password,
        cert_common_name=cert_common_name,
        verify_ssl=verify_ssl,
        cert=cert,
        params=params,
    )


def get_by_display_name(
    hostname, username, password, display_name, verify_ssl=True, cert=None, cert_common_name=None
):
    """
    Gets IP Address block present in the NSX-T Manager with given name.

    CLI Example:

    .. code-block:: bash

        salt vm_minion nsxt_ip_blocks.get_by_display_name hostname=nsxt-manager.local username=admin ...

    hostname
        The host name of NSX-T manager

    username
        Username to connect to NSX-T manager

    password
        Password to connect to NSX-T manager

    display_name
        The name of IP Address block to fetch

    verify_ssl
        (Optional) Option to enable/disable SSL verification. Enabled by default.
        If set to False, the certificate validation is skipped.

    cert
        (Optional) Path to the SSL client certificate file to connect to NSX-T manager.
        The certificate can be retrieved from browser.

    cert_common_name
        (Optional) By default, the hostname parameter and the common name in certificate is compared for host name
        verification. If the client certificate common name and hostname do not match (in case of self-signed
        certificates), specify the certificate common name as part of this parameter. This value is then used to
        compare against

    """

    log.info("Finding IP Address Blocks with display name: %s", display_name)

    ip_blocks = common._read_paginated(
        func=get,
        display_name=display_name,
        hostname=hostname,
        username=username,
        password=password,
        verify_ssl=verify_ssl,
        cert=cert,
        cert_common_name=cert_common_name,
    )

    if "error" in ip_blocks:
        return ip_blocks

    return {"results": ip_blocks}


def create(
    cidr,
    hostname,
    username,
    password,
    verify_ssl=True,
    cert=None,
    cert_common_name=None,
    display_name=None,
    description=None,
    tags=None,
):
    """
    Creates an IP Address block with given specifications

    CLI Example:

    .. code-block:: bash

        salt vm_minion nsxt_ip_blocks.create hostname=nsxt-manager.local username=admin ...

    hostname
        The host name of NSX-T manager

    username
        Username to connect to NSX-T manager

    password
        Password to connect to NSX-T manager

    verify_ssl
        (Optional) Option to enable/disable SSL verification. Enabled by default.
        If set to False, the certificate validation is skipped.

    cert
        (Optional) Path to the SSL client certificate file to connect to NSX-T manager.
        The certificate can be retrieved from browser.

    cert_common_name
        (Optional) By default, the hostname parameter and the common name in certificate is compared for host name
        verification. If the client certificate common name and hostname do not match (in case of self-signed
        certificates), specify the certificate common name as part of this parameter. This value is then used to
        compare against

    cidr
        Represents network address and the prefix length which will be associated with a layer-2 broadcast domain

    display_name
        (Optional) The name using which IP Address Block will be created. If not provided then block id will be used as
        display name

    description
        (Optional) description for the IP Address Block

    tags
        (Optional) Opaque identifiers meaningful to the API user. Maximum 30 tags can be associated:

        .. code::

            tags='[
                {
                    "tag": "<tag-key-1>"
                    "scope": "<tag-value-1>"
                },
                {
                    "tag": "<tag-key-2>"
                    "scope": "<tag-value-2>"
                }
            ]'

    """

    log.info("Creating IP Address Block")
    url = IP_BLOCKS_BASE_URL.format(hostname)

    req_data = common._filter_kwargs(
        allowed_kwargs=["display_name", "description", "tags"],
        default_dict={"cidr": cidr},
        display_name=display_name,
        description=description,
        tags=tags,
    )

    return nsxt_request.call_api(
        method="post",
        url=url,
        username=username,
        password=password,
        cert_common_name=cert_common_name,
        verify_ssl=verify_ssl,
        cert=cert,
        data=req_data,
    )


def update(
    ip_block_id,
    cidr,
    display_name,
    revision,
    hostname,
    username,
    password,
    verify_ssl=True,
    cert=None,
    cert_common_name=None,
    description=None,
    tags=None,
):
    """
    Updates an IP Address block of display name with given specifications. All the fields for which no value is
    provided will be set to null

    CLI Example:

    .. code-block:: bash

        salt vm_minion nsxt_ip_blocks.update hostname=nsxt-manager.local username=admin ...

    hostname
        The host name of NSX-T manager

    username
        Username to connect to NSX-T manager

    password
        Password to connect to NSX-T manager

    verify_ssl
        (Optional) Option to enable/disable SSL verification. Enabled by default.
        If set to False, the certificate validation is skipped.

    cert
        (Optional) Path to the SSL client certificate file to connect to NSX-T manager.
        The certificate can be retrieved from browser.

    cert_common_name
        (Optional) By default, the hostname parameter and the common name in certificate is compared for host name
        verification. If the client certificate common name and hostname do not match (in case of self-signed
        certificates), specify the certificate common name as part of this parameter. This value is then used to
        compare against

    ip_block_id
        Id of the existing IP Address block

    display_name
        Existing IP Block display name. This is a non updatable field

    description
        (Optional) description for the IP Address Block

    tags
        (Optional) Opaque identifiers meaningful to the API user. Maximum 30 tags can be associated:

        .. code::

            tags='[
                {
                    "tag": "<tag-key-1>"
                    "scope": "<tag-value-1>"
                },
                {
                    "tag": "<tag-key-2>"
                    "scope": "<tag-value-2>"
                }
            ]'

    cidr
        Represents network address and the prefix length which will be associated with a layer-2 broadcast domain

    revision
        Revision number of IP block to update
    """

    log.info("Updating IP Address block %s", display_name)
    url = IP_BLOCKS_BASE_URL.format(hostname) + "/{}".format(ip_block_id)

    req_data = common._filter_kwargs(
        allowed_kwargs=["description", "tags"],
        default_dict={
            "id": ip_block_id,
            "_revision": revision,
            "display_name": display_name,
            "cidr": cidr,
        },
        tags=tags,
        description=description,
    )

    return nsxt_request.call_api(
        method="put",
        url=url,
        username=username,
        password=password,
        cert_common_name=cert_common_name,
        verify_ssl=verify_ssl,
        cert=cert,
        data=req_data,
    )


def delete(
    ip_block_id, hostname, username, password, verify_ssl=True, cert=None, cert_common_name=None
):
    """
    Deletes an IP Address block with given id

    CLI Example:

    .. code-block:: bash

        salt vm_minion nsxt_ip_blocks.delete hostname=nsxt-manager.local username=admin ...

    hostname
        The host name of NSX-T manager

    username
        Username to connect to NSX-T manager

    password
        Password to connect to NSX-T manager

    ip_block_id
        Existing IP Block id

    verify_ssl
        (Optional) Option to enable/disable SSL verification. Enabled by default.
        If set to False, the certificate validation is skipped.

    cert
        (Optional) Path to the SSL client certificate file to connect to NSX-T manager.
        The certificate can be retrieved from browser.

    cert_common_name
        (Optional) By default, the hostname parameter and the common name in certificate is compared for host name
        verification. If the client certificate common name and hostname do not match (in case of self-signed
        certificates), specify the certificate common name as part of this parameter. This value is then used to
        compare against

    """

    log.info("Deleting IP Address Block %s", ip_block_id)

    url = IP_BLOCKS_BASE_URL.format(hostname) + "/{}".format(ip_block_id)

    response = nsxt_request.call_api(
        method="delete",
        url=url,
        username=username,
        password=password,
        cert_common_name=cert_common_name,
        verify_ssl=verify_ssl,
        cert=cert,
    )

    return response or "IP Block deleted successfully"



"""
VMC Public IP state module

Add new public IP, update name of an existing public IP and delete existing public IP from an SDDC.

Example usage :

.. code-block:: yaml

    TEST_PUBLIC_IP:
      vmc_public_ip.present:
        - name: TEST_PUBLIC_IP
        - hostname: sample-nsx.vmwarevmc.com
        - refresh_key: 7jPSGSZpCa8e5Ouks4UY5cZyOtynshs
        - authorization_host: console-stg.cloud.vmware.com
        - org_id: 10e1092f-51d0-473a-80f8-137652c39fd0
        - sddc_id: b43da080-2626-f64c-88e8-7f31d9d2c306
        - verify_ssl: False
        - cert: /path/to/client/certificate

.. warning::

    It is recommended to pass the VMC authentication details using Pillars rather than specifying as plain text in SLS
    files.

"""
import logging

from saltext.vmware.utils import vmc_state

log = logging.getLogger(__name__)


def present(
    name,
    hostname,
    refresh_key,
    authorization_host,
    org_id,
    sddc_id,
    verify_ssl=True,
    cert=None,
    display_name=None,
):
    """
    Ensure a given public IP exists for given SDDC.

    name
        Indicates the ID and name that the public IP will be created with, any unique string identifying the public IP.

    hostname
        The host name of NSX-T manager.

    refresh_key
        API Token of the user which is used to get the Access Token required for VMC operations.

    authorization_host
        Hostname of the VMC cloud console.

    org_id
        The ID of organization to which the SDDC belongs to.

    sddc_id
        The ID of SDDC to which public IP should belong to.

    verify_ssl
        (Optional) Option to enable/disable SSL verification. Enabled by default.
        If set to False, the certificate validation is skipped.

    cert
        (Optional) Path to the SSL client certificate file to connect to VMC Cloud Console.
        The certificate can be retrieved from browser.

    display_name
        Indicates the new name of the public IP. This is required if you want to update name of the existing public IP.

    """
    public_ip_id = name

    public_ip_response = __salt__["vmc_public_ip.get"](
        id=public_ip_id,
        hostname=hostname,
        refresh_key=refresh_key,
        authorization_host=authorization_host,
        org_id=org_id,
        sddc_id=sddc_id,
        verify_ssl=verify_ssl,
        cert=cert,
    )

    if "error" in public_ip_response:
        if "PublicIp Object Not Found" in public_ip_response["error"]:
            public_ip_response = None
        else:
            return vmc_state._create_state_response(
                name=name, comment=public_ip_response["error"], result=False
            )

    if __opts__.get("test"):
        log.info("present is called with test option")
        return vmc_state._create_state_response(
            name=name,
            comment="Public IP {} would have been {}".format(
                public_ip_id, "updated" if public_ip_response else "created"
            ),
        )

    if public_ip_response:
        is_update_required = display_name and public_ip_response.get("display_name") != display_name
        if is_update_required:
            updated_public_ip = __salt__["vmc_public_ip.update"](
                id=public_ip_id,
                display_name=display_name,
                hostname=hostname,
                refresh_key=refresh_key,
                authorization_host=authorization_host,
                org_id=org_id,
                sddc_id=sddc_id,
                verify_ssl=verify_ssl,
                cert=cert,
            )

            if "error" in updated_public_ip:
                return vmc_state._create_state_response(
                    name=name, comment=updated_public_ip["error"], result=False
                )

            return vmc_state._create_state_response(
                name=name,
                comment="Updated public IP {}".format(public_ip_id),
                old_state=public_ip_response,
                new_state=updated_public_ip,
                result=True,
            )
        else:
            log.info("All fields are same as existing public IP %s", public_ip_id)
            return vmc_state._create_state_response(
                name=name, comment="Public IP exists already, no action to perform", result=True
            )
    else:
        created_public_ip = __salt__["vmc_public_ip.create"](
            id=public_ip_id,
            hostname=hostname,
            refresh_key=refresh_key,
            authorization_host=authorization_host,
            org_id=org_id,
            sddc_id=sddc_id,
            verify_ssl=verify_ssl,
            cert=cert,
        )

        if "error" in created_public_ip:
            return vmc_state._create_state_response(
                name=name, comment=created_public_ip["error"], result=False
            )

        return vmc_state._create_state_response(
            name=name,
            comment="Created public IP {}".format(public_ip_id),
            new_state=created_public_ip,
            result=True,
        )


def absent(
    name,
    hostname,
    refresh_key,
    authorization_host,
    org_id,
    sddc_id,
    verify_ssl=True,
    cert=None,
):
    """
    Ensure a given public IP does not exist on given SDDC.

    name
        Indicates the public IP ID, any unique string identifying the public IP.

    hostname
        The host name of NSX-T manager.

    refresh_key
        API Token of the user which is used to get the Access Token required for VMC operations.

    authorization_host
        Hostname of the VMC cloud console.

    org_id
        The ID of organization to which the SDDC belongs to.

    sddc_id
        The ID of SDDC to which the public IP belongs to.

    verify_ssl
        (Optional) Option to enable/disable SSL verification. Enabled by default.
        If set to False, the certificate validation is skipped.

    cert
        (Optional) Path to the SSL client certificate file to connect to VMC Cloud Console.
        The certificate can be retrieved from browser.

    """
    public_ip_id = name
    log.info("Checking if public IP with ID %s is present", public_ip_id)
    public_ip_response = __salt__["vmc_public_ip.get"](
        id=public_ip_id,
        hostname=hostname,
        refresh_key=refresh_key,
        authorization_host=authorization_host,
        org_id=org_id,
        sddc_id=sddc_id,
        verify_ssl=verify_ssl,
        cert=cert,
    )

    if "error" in public_ip_response:
        if "PublicIp Object Not Found" in public_ip_response["error"]:
            public_ip_response = None
        else:
            return vmc_state._create_state_response(
                name=name, comment=public_ip_response["error"], result=False
            )

    if __opts__.get("test"):
        log.info("absent is called with test option")
        if public_ip_response:
            return vmc_state._create_state_response(
                name=name,
                comment="State absent will delete public IP with ID {}".format(public_ip_id),
            )
        else:
            return vmc_state._create_state_response(
                name=name,
                comment="State absent will do nothing as no public IP found with ID {}".format(
                    public_ip_id
                ),
            )

    if public_ip_response:
        log.info("Public IP found with ID %s", public_ip_id)
        deleted_public_ip = __salt__["vmc_public_ip.delete"](
            id=public_ip_id,
            hostname=hostname,
            refresh_key=refresh_key,
            authorization_host=authorization_host,
            org_id=org_id,
            sddc_id=sddc_id,
            verify_ssl=verify_ssl,
            cert=cert,
        )

        if "error" in deleted_public_ip:
            return vmc_state._create_state_response(
                name=name, comment=deleted_public_ip["error"], result=False
            )

        return vmc_state._create_state_response(
            name=name,
            comment="Deleted public IP {}".format(public_ip_id),
            old_state=public_ip_response,
            result=True,
        )
    else:
        log.info("No public IP found with ID %s", public_ip_id)
        return vmc_state._create_state_response(
            name=name, comment="No public IP found with ID {}".format(public_ip_id), result=True
        )



"""
VMC DHCP Profiles state module

Add new DHCP profile and delete existing DHCP profile from an SDDC.

Example usage :

.. code-block:: yaml

    ensure_dhcp_profile:
      vmc_dhcp_profiles.present:
        - hostname: sample-nsx.vmwarevmc.com
        - refresh_key: 7jPSGSZpCa8e5Ouks4UY5cZyOtynAhF
        - authorization_host: console-stg.cloud.vmware.com
        - org_id: 10e1092f-51d0-473a-80f8-137652c39fd0
        - sddc_id: b43da080-2626-f64c-88e8-7f31d9d2c306
        - type: server
        - dhcp_profile_id: dhcp-test
        - verify_ssl: False
        - cert: /path/to/client/certificate

.. warning::

    It is recommended to pass the VMC authentication details using Pillars rather than specifying as plain text in SLS
    files.


"""
import logging

from saltext.vmware.utils import vmc_constants
from saltext.vmware.utils import vmc_request
from saltext.vmware.utils import vmc_state

log = logging.getLogger(__name__)


def present(
    name,
    hostname,
    refresh_key,
    authorization_host,
    org_id,
    sddc_id,
    type,
    dhcp_profile_id,
    verify_ssl=True,
    cert=None,
    server_addresses=None,
    tags=vmc_constants.VMC_NONE,
    lease_time=None,
    display_name=None,
):
    """
    Ensure a given DHCP profile exists for given SDDC

    hostname
        The host name of NSX-T manager

    refresh_key
        API Token of the user which is used to get the Access Token required for VMC operations

    authorization_host
        Hostname of the VMC cloud console

    org_id
        The Id of organization to which the SDDC belongs to

    sddc_id
        The Id of SDDC for which the DHCP profile should be added

    type
        The type of DHCP profile for which the given dhcp belongs to. Possible values: server, relay

    dhcp_profile_id
        DHCP profile id, any static unique string identifying the DHCP profile.
        Also same as the display_name by default.

    verify_ssl
        (Optional) Option to enable/disable SSL verification. Enabled by default.
        If set to False, the certificate validation is skipped.

    cert
        (Optional) Path to the SSL client certificate file to connect to VMC Cloud Console.
        The certificate can be retrieved from browser.

    server_addresses
        when type is relay, this field indicates DHCP relay addresses(DHCP server IP addresses
        for DHCP relay configuration).
        Both IPv4 and IPv6 addresses are supported.

        when type is server, this field indicates DHCP server address in CIDR format.
        Both IPv4 and IPv6 address families are supported.
        Prefix length should be less than or equal to 30 for IPv4 address family
        and less than or equal to 126 for IPv6.
        When not specified, IPv4 value is auto-assigned to 100.96.0.1/30.

        Note: This field is optional only when type is server and
        is mandatory when type is relay.

    tags
        (Optional) Opaque identifiers meaningful to the user.

        .. code::

            tags:
              - tag: <tag-key-1>
                scope: <tag-value-1>
              - tag: <tag-key-2>
                scope: <tag-value-2>

    lease_time
        (Optional) IP address lease time in seconds. Minimum value is 60.
        Maximum value is 4294967295. Default value is 86400
        Note: This field is applicable only when type is server

    display_name
        Identifier to use when displaying entity in logs or GUI.

    Example values:

        .. code::

            for dhcp-server-profiles:

            server_addresses:
              - 203.0.113.2/23
            tags:
              - tag: tag1
                scope: scope1
            lease_time: 86400

            for dhcp-relay-profiles:

            server_addresses:
              - 203.0.113.1
            tags:
              - tag: tag1
                scope: scope1

    """

    input_dict = {}
    if type == vmc_constants.RELAY:
        if lease_time:
            return vmc_state._create_state_response(
                name=name,
                comment="lease_time is not applicable for DHCP Relay Profile",
                result=False,
            )
    else:
        if lease_time != vmc_constants.VMC_NONE:
            input_dict["lease_time"] = lease_time

    input_dict.update(
        vmc_request._filter_vmc_none(
            server_addresses=server_addresses,
            tags=tags,
            display_name=display_name,
        )
    )

    dhcp_profile = __salt__["vmc_dhcp_profiles.get_by_id"](
        hostname=hostname,
        refresh_key=refresh_key,
        authorization_host=authorization_host,
        org_id=org_id,
        sddc_id=sddc_id,
        type=type,
        dhcp_profile_id=dhcp_profile_id,
        verify_ssl=verify_ssl,
        cert=cert,
    )

    if "error" in dhcp_profile:
        if "could not be found" in dhcp_profile["error"]:
            dhcp_profile = None
        else:
            return vmc_state._create_state_response(
                name=name, comment=dhcp_profile["error"], result=False
            )

    if __opts__.get("test"):
        log.info("present is called with test option")
        if dhcp_profile:
            return vmc_state._create_state_response(
                name=name,
                comment="State present will update DHCP Profile {}".format(dhcp_profile_id),
            )
        else:
            return vmc_state._create_state_response(
                name=name,
                comment="State present will create DHCP Profile {}".format(dhcp_profile_id),
            )

    if dhcp_profile:
        is_update_required = vmc_state._check_for_updates(dhcp_profile, input_dict, None, ["tags"])

        if is_update_required:
            updated_dhcp_profile = __salt__["vmc_dhcp_profiles.update"](
                hostname=hostname,
                refresh_key=refresh_key,
                authorization_host=authorization_host,
                org_id=org_id,
                sddc_id=sddc_id,
                type=type,
                dhcp_profile_id=dhcp_profile_id,
                verify_ssl=verify_ssl,
                cert=cert,
                server_addresses=server_addresses,
                tags=tags,
                lease_time=lease_time,
                display_name=display_name,
            )

            if "error" in updated_dhcp_profile:
                return vmc_state._create_state_response(
                    name=name, comment=updated_dhcp_profile["error"], result=False
                )

            updated_dhcp_profile = __salt__["vmc_dhcp_profiles.get_by_id"](
                hostname=hostname,
                refresh_key=refresh_key,
                authorization_host=authorization_host,
                org_id=org_id,
                sddc_id=sddc_id,
                type=type,
                dhcp_profile_id=dhcp_profile_id,
                verify_ssl=verify_ssl,
                cert=cert,
            )

            if "error" in updated_dhcp_profile:
                return vmc_state._create_state_response(
                    name=name, comment=updated_dhcp_profile["error"], result=False
                )

            return vmc_state._create_state_response(
                name=name,
                comment="Updated DHCP Profile {}".format(dhcp_profile_id),
                old_state=dhcp_profile,
                new_state=updated_dhcp_profile,
                result=True,
            )
        else:
            log.info("All fields are same as existing DHCP Profile %s", dhcp_profile_id)
            return vmc_state._create_state_response(
                name=name, comment="DHCP Profile exists already, no action to perform", result=True
            )
    else:
        log.info("No DHCP Profile found with Id %s", dhcp_profile_id)
        created_dhcp_profile = __salt__["vmc_dhcp_profiles.create"](
            hostname=hostname,
            refresh_key=refresh_key,
            authorization_host=authorization_host,
            org_id=org_id,
            sddc_id=sddc_id,
            type=type,
            dhcp_profile_id=dhcp_profile_id,
            verify_ssl=verify_ssl,
            cert=cert,
            server_addresses=server_addresses,
            tags=tags,
            lease_time=lease_time,
        )

        if "error" in created_dhcp_profile:
            return vmc_state._create_state_response(
                name=name, comment=created_dhcp_profile["error"], result=False
            )

        return vmc_state._create_state_response(
            name=name,
            comment="Created DHCP Profile {}".format(dhcp_profile_id),
            new_state=created_dhcp_profile,
            result=True,
        )


def absent(
    name,
    hostname,
    refresh_key,
    authorization_host,
    org_id,
    sddc_id,
    type,
    dhcp_profile_id,
    verify_ssl=True,
    cert=None,
):
    """
    Ensure a given DHCP profile does not exist on given SDDC

    hostname
        The host name of NSX-T manager

    refresh_key
        API Token of the user which is used to get the Access Token required for VMC operations

    authorization_host
        Hostname of the VMC cloud console

    org_id
        The Id of organization to which the SDDC belongs to

    sddc_id
        The Id of SDDC from which the DHCP profile should be deleted

    type
        The type of DHCP profile for which the given dhcp belongs to. Possible values: server, relay

    dhcp_profile_id
        DHCP profile id, any static unique string identifying the DHCP profile.

    verify_ssl
        (Optional) Option to enable/disable SSL verification. Enabled by default.
        If set to False, the certificate validation is skipped.

    cert
        (Optional) Path to the SSL client certificate file to connect to VMC Cloud Console.
        The certificate can be retrieved from browser.
    """
    log.info("Checking if DHCP Profile with Id %s is present", dhcp_profile_id)
    dhcp_profile = __salt__["vmc_dhcp_profiles.get_by_id"](
        hostname=hostname,
        refresh_key=refresh_key,
        authorization_host=authorization_host,
        org_id=org_id,
        sddc_id=sddc_id,
        type=type,
        dhcp_profile_id=dhcp_profile_id,
        verify_ssl=verify_ssl,
        cert=cert,
    )

    if "error" in dhcp_profile:
        if "could not be found" in dhcp_profile["error"]:
            dhcp_profile = None
        else:
            return vmc_state._create_state_response(
                name=name, comment=dhcp_profile["error"], result=False
            )

    if __opts__.get("test"):
        log.info("absent is called with test option")
        if dhcp_profile:
            return vmc_state._create_state_response(
                name=name,
                comment="State absent will delete DHCP Profile with Id {}".format(dhcp_profile_id),
            )
        else:
            return vmc_state._create_state_response(
                name=name,
                comment="State absent will do nothing as no DHCP Profile found with Id {}".format(
                    dhcp_profile_id
                ),
            )

    if dhcp_profile:
        log.info("DHCP Profile found with Id %s", dhcp_profile_id)
        deleted_dhcp_profile = __salt__["vmc_dhcp_profiles.delete"](
            hostname=hostname,
            refresh_key=refresh_key,
            authorization_host=authorization_host,
            org_id=org_id,
            sddc_id=sddc_id,
            type=type,
            dhcp_profile_id=dhcp_profile_id,
            verify_ssl=verify_ssl,
            cert=cert,
        )

        if "error" in deleted_dhcp_profile:
            return vmc_state._create_state_response(
                name=name, comment=deleted_dhcp_profile["error"], result=False
            )

        return vmc_state._create_state_response(
            name=name,
            comment="Deleted DHCP Profile {}".format(dhcp_profile_id),
            old_state=dhcp_profile,
            result=True,
        )
    else:
        log.info("No DHCP Profile found with Id %s", dhcp_profile_id)
        return vmc_state._create_state_response(
            name=name,
            comment="No DHCP Profile found with Id {}".format(dhcp_profile_id),
            result=True,
        )
